@file:OptIn(ExperimentalMaterial3Api::class)

package com.example.app.screens

import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.runtime.Composable
import androidx.compose.runtime.remember
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import com.arcgismaps.Color
import com.arcgismaps.geometry.Point
import com.arcgismaps.geometry.PolygonBuilder
import com.arcgismaps.geometry.PolylineBuilder
import com.arcgismaps.geometry.SpatialReference
import com.arcgismaps.mapping.ArcGISMap
import com.arcgismaps.mapping.BasemapStyle
import com.arcgismaps.mapping.Viewpoint
import com.arcgismaps.mapping.symbology.SimpleFillSymbol
import com.arcgismaps.mapping.symbology.SimpleFillSymbolStyle
import com.arcgismaps.mapping.symbology.SimpleLineSymbol
import com.arcgismaps.mapping.symbology.SimpleLineSymbolStyle
import com.arcgismaps.mapping.symbology.SimpleMarkerSymbol
import com.arcgismaps.mapping.symbology.SimpleMarkerSymbolStyle
import com.arcgismaps.mapping.view.Graphic
import com.arcgismaps.mapping.view.GraphicsOverlay
import com.arcgismaps.toolkit.geoviewcompose.MapView
import com.example.app.R

// Create a blue outline symbol.
private val blueOutlineSymbol by lazy {
    SimpleLineSymbol(SimpleLineSymbolStyle.Solid, Color.fromRgba(0, 0, 255), 2f)
}

private val pointGraphic by lazy {

    // Create a point geometry with a location and spatial reference.
    // Point(latitude, longitude, spatial reference)
    val point = Point(
        x = -118.8065,
        y = 34.0005,
        spatialReference = SpatialReference.wgs84()
    )

    // Create a point symbol that is an small red circle and assign the blue outline symbol to its outline property.
    val simpleMarkerSymbol = SimpleMarkerSymbol(
        style = SimpleMarkerSymbolStyle.Circle,
        color = Color.red,
        size = 10f
    )
    simpleMarkerSymbol.outline = blueOutlineSymbol

    // Create a graphic with the point geometry and symbol.
    Graphic(
        geometry = point,
        symbol = simpleMarkerSymbol
    )

}

private val polylineGraphic by lazy {

    // Create a blue line symbol for the polyline.
    val polylineSymbol = SimpleLineSymbol(
        style = SimpleLineSymbolStyle.Solid,
        color = Color.fromRgba(0, 0, 255),
        width = 3f
    )

    // Create a polylineBuilder with a spatial reference and add three points to it.
    val polylineBuilder = PolylineBuilder(SpatialReference.wgs84()) {
        addPoint(-118.8215, 34.0139)
        addPoint(-118.8148, 34.0080)
        addPoint(-118.8088, 34.0016)
    }
    // then get the polyline from the polyline builder
    val polyline = polylineBuilder.toGeometry()

    // Create a polyline graphic with the polyline geometry and symbol.
    Graphic(
        geometry = polyline,
        symbol = polylineSymbol
    )

}

private val polygonGraphic by lazy {

    // Create a polygon builder with a spatial reference and add five vertices (points) to it.
    // Then get the polygon from the polygon builder.
    val polygonBuilder = PolygonBuilder(SpatialReference.wgs84()) {
        addPoint(-118.8189, 34.0137)
        addPoint(-118.8067, 34.0215)
        addPoint(-118.7914, 34.0163)
        addPoint(-118.7959, 34.0085)
        addPoint(-118.8085, 34.0035)
    }
    val polygon = polygonBuilder.toGeometry()

    // Create a red fill symbol with an alpha component of 128: values can run from 0 to 255).
    val polygonFillSymbol = SimpleFillSymbol(
        style = SimpleFillSymbolStyle.Solid,
        color = Color.fromRgba(255, 0, 0, 128),
        outline = blueOutlineSymbol
    )

    // Create a polygon graphic from the polygon geometry and symbol.
    Graphic(
        geometry = polygon,
        symbol = polygonFillSymbol
    )

}

@Composable
fun MainScreen() {
    // create an ArcGISMap
    val map = remember {
        createMap()
    }

    // Create a graphics overlay.
    val graphicsOverlay = remember { GraphicsOverlay() }

    // Add the point graphic to the graphics overlay.
    graphicsOverlay.graphics.add(pointGraphic)

    // Add the polyline graphic to the graphics overlay.
    graphicsOverlay.graphics.add(polylineGraphic)

    // Add the polygon graphic to the graphics overlay.
    graphicsOverlay.graphics.add(polygonGraphic)

    // Create a list of graphics overlays used by the MapView
    val graphicsOverlays = remember { listOf(graphicsOverlay) }

    Scaffold(
        topBar = { TopAppBar(title = { Text(text = stringResource(id = R.string.app_name)) }) }
    ) {
        MapView(
            modifier = Modifier.fillMaxSize().padding(it),
            arcGISMap = map,

            graphicsOverlays = graphicsOverlays

        )
    }

}

fun createMap(): ArcGISMap {
    return ArcGISMap(BasemapStyle.ArcGISTopographic).apply {
        initialViewpoint = Viewpoint(
            latitude = 34.0270,
            longitude = -118.8050,
            scale = 72000.0
        )
    }
}

